'use strict';

exports.__esModule = true;

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _map = require('babel-runtime/core-js/map');

var _map2 = _interopRequireDefault(_map);

var _path = require('path');

var path = _interopRequireWildcard(_path);

var _arrayIncludes = require('array-includes');

var _arrayIncludes2 = _interopRequireDefault(_arrayIncludes);

var _semver = require('semver');

var _semver2 = _interopRequireDefault(_semver);

var _Package = require('./Package');

var _Package2 = _interopRequireDefault(_Package);

var _Workspace = require('./Workspace');

var _Workspace2 = _interopRequireDefault(_Workspace);

var _Config = require('./Config');

var _Config2 = _interopRequireDefault(_Config);

var _fs = require('./utils/fs');

var fs = _interopRequireWildcard(_fs);

var _logger = require('./utils/logger');

var logger = _interopRequireWildcard(_logger);

var _messages = require('./utils/messages');

var messages = _interopRequireWildcard(_messages);

var _errors = require('./utils/errors');

var _globs = require('./utils/globs');

var globs = _interopRequireWildcard(_globs);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Project {

  constructor(pkg) {
    this.pkg = pkg;
  }

  static async init(cwd) {
    let filePath = await _Config2.default.getProjectConfig(cwd);
    if (!filePath) throw new _errors.BoltError(`Unable to find root of project in ${cwd}`);
    let pkg = await _Package2.default.init(filePath);
    return new Project(pkg);
  }

  async getWorkspaces() {
    let queue = [this.pkg];
    let workspaces = [];

    for (let pkg of queue) {
      let cwd = path.dirname(pkg.filePath);
      let patterns = pkg.getWorkspacesConfig();
      let matchedPaths = await globs.findWorkspaces(cwd, patterns);

      for (let matchedPath of matchedPaths) {
        let dir = path.join(cwd, matchedPath);
        let stats = await fs.stat(dir);
        if (!stats.isDirectory()) continue;

        let filePath = path.join(dir, 'package.json');
        let wPkg = await _Package2.default.init(filePath);
        let workspace = await _Workspace2.default.init(wPkg);

        queue.push(wPkg);
        workspaces.push(workspace);
      }
    }

    return workspaces;
  }

  async getDependencyGraph(workspaces) {
    let graph = new _map2.default();
    let packages = [this.pkg];
    let packagesByName = { [this.pkg.config.getName()]: this.pkg };
    let valid = true;

    for (let workspace of workspaces) {
      packages.push(workspace.pkg);
      packagesByName[workspace.pkg.config.getName()] = workspace.pkg;
    }

    for (let pkg of packages) {
      let name = pkg.config.getName();
      let dependencies = [];
      let allDependencies = pkg.getAllDependencies();

      for (let [depName, depVersion] of allDependencies) {
        let match = packagesByName[depName];
        if (!match) continue;

        let actual = depVersion;
        let expected = match.config.getVersion();

        // Workspace dependencies only need to semver satisfy, not '==='
        if (!_semver2.default.satisfies(expected, depVersion)) {
          valid = false;
          logger.error(messages.packageMustDependOnCurrentVersion(name, depName, expected, depVersion));
          continue;
        }

        dependencies.push(depName);
      }

      graph.set(name, { pkg, dependencies });
    }

    return { graph, valid };
  }

  async getDependentsGraph(workspaces) {
    const graph = new _map2.default();
    const { valid, graph: dependencyGraph } = await this.getDependencyGraph(workspaces);

    const dependentsLookup = {};

    workspaces.forEach(workspace => {
      dependentsLookup[workspace.pkg.config.getName()] = {
        pkg: workspace.pkg,
        dependents: []
      };
    });

    workspaces.forEach(workspace => {
      const dependent = workspace.pkg.config.getName();
      const valFromDependencyGraph = dependencyGraph.get(dependent) || {};
      const dependencies = valFromDependencyGraph.dependencies || [];

      dependencies.forEach(dependency => {
        dependentsLookup[dependency].dependents.push(dependent);
      });
    });

    // can't use Object.entries here as the flow type for it is Array<[string, mixed]>;
    (0, _keys2.default)(dependentsLookup).forEach(key => {
      graph.set(key, dependentsLookup[key]);
    });

    return { valid, graph };
  }

  // TODO: Properly sort packages using a topological sort, resolving cycles
  // with groups specified in `package.json#pworkspaces`
  static async runWorkspaceTasks(workspaces, task) {
    let promises = [];

    for (let workspace of workspaces) {
      promises.push(task(workspace));
    }

    await _promise2.default.all(promises);
  }

  getWorkspaceByName(workspaces, workspaceName) {
    return workspaces.find(workspace => {
      return workspace.pkg.config.getName() === workspaceName;
    });
  }

  filterWorkspaces(workspaces, opts) {
    let relativeDir = workspace => path.relative(this.pkg.dir, workspace.pkg.dir);

    let workspaceNames = workspaces.map(ws => ws.pkg.config.getName());
    let workspaceDirs = workspaces.map(ws => relativeDir(ws));

    let filteredByName = globs.matchOnlyAndIgnore(workspaceNames, opts.only, opts.ignore);

    let filteredByDir = globs.matchOnlyAndIgnore(workspaceDirs, opts.onlyFs, opts.ignoreFs);

    let filteredWorkspaces = workspaces.filter(workspace => (0, _arrayIncludes2.default)(filteredByName, workspace.pkg.config.getName()) && (0, _arrayIncludes2.default)(filteredByDir, relativeDir(workspace)));

    if (filteredWorkspaces.length === 0) {
      logger.warn(messages.noPackagesMatchFilters());
    }

    return filteredWorkspaces;
  }
}
exports.default = Project;