'use strict';

exports.__esModule = true;
exports.info = info;
exports.infoAllow404 = infoAllow404;
exports.publish = publish;
exports.addTag = addTag;
exports.removeTag = removeTag;

var _errors = require('./errors');

var _logger = require('./logger');

var logger = _interopRequireWildcard(_logger);

var _messages = require('./messages');

var messages = _interopRequireWildcard(_messages);

var _processes = require('./processes');

var processes = _interopRequireWildcard(_processes);

var _pLimit = require('p-limit');

var _pLimit2 = _interopRequireDefault(_pLimit);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

const npmRequestLimit = (0, _pLimit2.default)(40);
function info(pkgName) {
  return npmRequestLimit(async () => {
    logger.info(messages.npmInfo(pkgName));

    const result = await processes.spawn('npm', ['info', pkgName, '--json'], {
      silent: true
    });

    return JSON.parse(result.stdout);
  });
}

async function infoAllow404(pkgName) {
  try {
    const pkgInfo = await info(pkgName);
    return { published: true, pkgInfo };
  } catch (error) {
    const output = JSON.parse(error.stdout);
    if (output.error && output.error.code === 'E404') {
      logger.warn(messages.npmInfo404(pkgName));
      return { published: false, pkgInfo: {} };
    }
    throw error;
  }
}

function publish(pkgName, opts = {}) {
  return npmRequestLimit(async () => {
    logger.info(messages.npmPublish(pkgName));
    let publishFlags = opts.access ? ['--access', opts.access] : [];
    return await processes.spawn('npm', ['publish', ...publishFlags], {
      cwd: opts.cwd
    });
  });
}

function addTag(pkgName, pkgVersion, tag) {
  return npmRequestLimit(async () => {
    logger.info(messages.npmDistTagAdd(pkgName, pkgVersion, tag));
    let pkgStr = `${pkgName}@${pkgVersion}`;
    let result = await processes.spawn('npm', ['dist-tag', 'add', pkgStr, tag]);
    // An existing tag will return a warning to stderr, but a 0 status code
    if (result.stderr) {
      throw new _errors.BoltError(`Could not add tag ${tag} to ${pkgStr} as one already exists`);
    }
    return result;
  });
}

function removeTag(pkgName, tag) {
  return npmRequestLimit(async () => {
    logger.info(messages.npmDistTagRm(pkgName, tag));

    try {
      return await processes.spawn('npm', ['dist-tag', 'rm', pkgName, tag], {
        silent: true
      });
    } catch (error) {
      // The dist tag not existing is unexpected, but shouldn't prevent execution
      if (error.code === 1 && error.stderr.includes('is not a dist-tag on')) {
        logger.warn(messages.notDistTagFound(tag, pkgName));
        return;
      } else if (error.stderr && error.stderr.startsWith('npm ERR! code E404')) {
        // the package does not exist yet, warn but dont error
        logger.warn(messages.npmPackageCouldNotBeFound(pkgName));
        return;
      }
      throw error;
    }
  });
}